@@ -152,9 +152,9 @@ eog_job_thumbnail_dispose (GObject *object)
 
 	job = EOG_JOB_THUMBNAIL (object);
 
-	if (job->uri_entry) {
-		gnome_vfs_uri_unref (job->uri_entry);
-		job->uri_entry = NULL;
+	if (job->image) {
+		g_object_unref (job->image);
+		job->image = NULL;
 	}
 	
 	if (job->thumbnail) {
@@ -176,15 +176,14 @@ eog_job_thumbnail_class_init (EogJobThumbnailClass *class)
 }
 
 EogJob *
-eog_job_thumbnail_new (GnomeVFSURI *uri_entry)
+eog_job_thumbnail_new (EogImage *image)
 {
 	EogJobThumbnail *job;
 
 	job = g_object_new (EOG_TYPE_JOB_THUMBNAIL, NULL);
 
-	if (uri_entry) {
-		gnome_vfs_uri_ref (uri_entry);
-		job->uri_entry = uri_entry;
+	if (image) {
+		job->image = g_object_ref (image);
 	}
 
 	return EOG_JOB (job);
@@ -193,7 +192,6 @@ eog_job_thumbnail_new (GnomeVFSURI *uri_entry)
 void
 eog_job_thumbnail_run (EogJobThumbnail *job)
 {
-	gint width, height;
 	gchar *orig_width, *orig_height;
 
 	g_return_if_fail (EOG_IS_JOB_THUMBNAIL (job));
@@ -203,15 +201,17 @@ eog_job_thumbnail_run (EogJobThumbnail *job)
 		EOG_JOB (job)->error = NULL;
 	}
 
-	job->thumbnail = eog_thumbnail_load (job->uri_entry,
+	job->thumbnail = eog_thumbnail_load (job->image,
 					     &EOG_JOB (job)->error);
 
+	if (!job->thumbnail) {
+		EOG_JOB (job)->finished = TRUE;
+		return;
+	}
+
 	orig_width = g_strdup (gdk_pixbuf_get_option (job->thumbnail, "tEXt::Thumb::Image::Width"));
 	orig_height = g_strdup (gdk_pixbuf_get_option (job->thumbnail, "tEXt::Thumb::Image::Height"));
 
-	width = gdk_pixbuf_get_width (job->thumbnail);
-	height = gdk_pixbuf_get_height (job->thumbnail);
-
 	eog_thumbnail_fit_to_size (&job->thumbnail, EOG_LIST_STORE_THUMB_SIZE);
 	eog_thumbnail_add_frame (&job->thumbnail);
 
@@ -229,7 +229,7 @@ eog_job_thumbnail_run (EogJobThumbnail *job)
 	}
 	
 	if (EOG_JOB (job)->error) {
-		g_warning ("%s\n", EOG_JOB (job)->error->message);
+		g_warning ("%s", EOG_JOB (job)->error->message);
 	}
 
 	EOG_JOB (job)->finished = TRUE;
