@@ -292,9 +292,12 @@ tb_on_button_press_event_cb (GtkWidget *tb, GdkEventButton *event,
 		    return FALSE;
 	    }
 
-	    gtk_icon_view_unselect_all (GTK_ICON_VIEW (tb));
-	    gtk_icon_view_select_path (GTK_ICON_VIEW (tb), path);
-
+	    if (!gtk_icon_view_path_is_selected (GTK_ICON_VIEW (tb), path) ||
+		eog_thumb_view_get_n_selected (EOG_THUMB_VIEW (tb)) != 1) {
+		gtk_icon_view_unselect_all (GTK_ICON_VIEW (tb));
+		gtk_icon_view_select_path (GTK_ICON_VIEW (tb), path);
+		gtk_icon_view_set_cursor (GTK_ICON_VIEW (tb), path, NULL, FALSE);
+	    }
 	    eog_thumb_view_popup_menu (EOG_THUMB_VIEW (tb), event);
 	    
 	    gtk_tree_path_free (path);
@@ -426,23 +429,28 @@ tb_on_query_tooltip_cb (GtkWidget  *widget,
 
 	type_str = gnome_vfs_mime_get_description (mime_str);
 
-	tooltip_string = g_strdup_printf ("<b><big>%s</big></b>\n"
-					  "%i x %i %s\n"
-					  "%s\n"
-					  "%s",
-					  eog_image_get_caption (image),
-					  width, 
-					  height, 
-					  ngettext ("pixel", "pixels", height), 
-					  bytes,
-					  type_str);
+	tooltip_string = g_markup_printf_escaped ("<b><big>%s</big></b>\n"
+						  "%i x %i %s\n"
+						  "%s\n"
+						  "%s",
+						  eog_image_get_caption (image),
+						  width, 
+						  height, 
+						  ngettext ("pixel",
+						  	    "pixels",
+							    height), 
+						  bytes,
+						  type_str);
 
 #ifdef HAVE_EXIF
 	if (exif_data) {
 		gchar *extra_info, *tmp, *date;
+		/* The EXIF standard says that the DATE_TIME tag is 
+		 * 20 bytes long. A 32-byte buffer should be large enough. */
+		gchar time_buffer[32];
 		
 		date = eog_exif_util_format_date (
-				eog_exif_util_get_value (exif_data, EXIF_TAG_DATE_TIME));
+				eog_exif_util_get_value (exif_data, EXIF_TAG_DATE_TIME_ORIGINAL, time_buffer, 32));
 
 		extra_info = g_strdup_printf ("\n%s %s", _("Taken on"), date);
 
@@ -536,16 +544,19 @@ eog_thumb_view_new (void)
 void
 eog_thumb_view_set_model (EogThumbView *tb, EogListStore *store)
 {
+	gint index;
+	
 	g_return_if_fail (EOG_IS_THUMB_VIEW (tb));
 	g_return_if_fail (EOG_IS_LIST_STORE (store));
 	
-	gint index = eog_list_store_get_initial_pos (store);
+	index = eog_list_store_get_initial_pos (store);
 
 	gtk_icon_view_set_model (GTK_ICON_VIEW (tb), GTK_TREE_MODEL (store));
 
 	if (index >= 0) {
 		GtkTreePath *path = gtk_tree_path_new_from_indices (index, -1);
 		gtk_icon_view_select_path (GTK_ICON_VIEW (tb), path);
+		gtk_icon_view_set_cursor (GTK_ICON_VIEW (tb), path, NULL, FALSE);
 		gtk_icon_view_scroll_to_path (GTK_ICON_VIEW (tb), path, FALSE, 0, 0);
 		gtk_tree_path_free (path);
 	}
@@ -604,19 +615,16 @@ eog_thumb_view_get_first_selected_image (EogThumbView *tb)
 	   smaller tree path value => tricky and expensive. Do we really need this?
 	*/
 	EogImage *image;
+	GtkTreePath *path;
 	GList *list = gtk_icon_view_get_selected_items (GTK_ICON_VIEW (tb));
 
 	if (list == NULL) {
 		return NULL;
 	}
 
-	GtkTreePath *path = (GtkTreePath *) (list->data);
+	path = (GtkTreePath *) (list->data);
 
-	/* debugging purposes */
-	gchar *text_path;
 	image = eog_thumb_view_get_image_from_path (tb, path);
-	text_path = gtk_tree_path_to_string (path);
-	g_free (text_path);
 
 	g_list_foreach (list, (GFunc) gtk_tree_path_free , NULL);
 	g_list_free (list);
@@ -667,6 +675,7 @@ eog_thumb_view_set_current_image (EogThumbView *tb, EogImage *image,
 	}
 	
 	gtk_icon_view_select_path (GTK_ICON_VIEW (tb), path);
+	gtk_icon_view_set_cursor (GTK_ICON_VIEW (tb), path, NULL, FALSE);
 	gtk_icon_view_scroll_to_path (GTK_ICON_VIEW (tb), path, FALSE, 0, 0);
 
 	gtk_tree_path_free (path);
@@ -676,12 +685,13 @@ void
 eog_thumb_view_select_single (EogThumbView *tb, 
 			      EogThumbViewSelectionChange change)
 {
-	g_return_if_fail (EOG_IS_THUMB_VIEW (tb));
-	GtkTreePath *path = NULL;
+  	GtkTreePath *path = NULL;
 	GtkTreeModel *model;
 	GList *list;
 	gint n_items;
 
+	g_return_if_fail (EOG_IS_THUMB_VIEW (tb));
+
 	model = gtk_icon_view_get_model (GTK_ICON_VIEW (tb));
 
 	n_items = eog_list_store_length (EOG_LIST_STORE (model));
@@ -692,6 +702,8 @@ eog_thumb_view_select_single (EogThumbView *tb,
 	
 	if (eog_thumb_view_get_n_selected (tb) == 0) {
 		switch (change) {
+		case EOG_THUMB_VIEW_SELECT_CURRENT:
+			break;
 		case EOG_THUMB_VIEW_SELECT_RIGHT:
 		case EOG_THUMB_VIEW_SELECT_FIRST:
 			path = gtk_tree_path_new_first ();
@@ -736,6 +748,7 @@ eog_thumb_view_select_single (EogThumbView *tb,
 	}
 
 	gtk_icon_view_select_path (GTK_ICON_VIEW (tb), path);
+	gtk_icon_view_set_cursor (GTK_ICON_VIEW (tb), path, NULL, FALSE);
 	gtk_icon_view_scroll_to_path (GTK_ICON_VIEW (tb), path, FALSE, 0, 0);
 	gtk_tree_path_free (path);
 }
